from unittest import TestCase

from calendar_view.core.data import CalendarConfig


class TestCalendarConfig(TestCase):
    def test_get_hours_range(self):
        self.assertEqual((0, 24), CalendarConfig(hours=None).get_hours_range())
        self.assertEqual((0, 24), CalendarConfig(hours='').get_hours_range())
        self.assertEqual((0, 24), CalendarConfig(hours='  ').get_hours_range())

        self.assertEqual((0, 24), CalendarConfig(hours='0 - 24').get_hours_range())
        self.assertEqual((0, 24), CalendarConfig(hours='0-24').get_hours_range())

        self.assertEqual((2, 23), CalendarConfig(hours='2 - 23').get_hours_range())
        self.assertEqual((4, 17), CalendarConfig(hours='4:00 - 17:00').get_hours_range())
        self.assertEqual((4, 18), CalendarConfig(hours='4:00 - 17:01').get_hours_range())

        self.assertRaises(ValueError, CalendarConfig(hours='8').get_hours_range)
        self.assertRaises(ValueError, CalendarConfig(hours='6:00').get_hours_range)
        self.assertRaises(ValueError, CalendarConfig(hours='4:00 - 4:00').get_hours_range)
        self.assertRaises(ValueError, CalendarConfig(hours='15:00 - 10:00').get_hours_range)
        self.assertRaises(ValueError, CalendarConfig(hours='15:00 - 14:55').get_hours_range)



from unittest import TestCase

from datetime import time, date, timedelta
from calendar_view.core import time_utils


class TestTimeUtils(TestCase):
    def test_is_valid_time(self):
        self.assertFalse(time_utils.is_valid_time(''))
        self.assertFalse(time_utils.is_valid_time('    '))
        self.assertFalse(time_utils.is_valid_time('zz'))
        self.assertFalse(time_utils.is_valid_time('55'))
        self.assertFalse(time_utils.is_valid_time('25'))
        self.assertFalse(time_utils.is_valid_time('-1'))
        self.assertFalse(time_utils.is_valid_time('24:01'))

        self.assertTrue(time_utils.is_valid_time('10:00'))
        self.assertTrue(time_utils.is_valid_time('00:00'))
        self.assertTrue(time_utils.is_valid_time('0'))
        self.assertTrue(time_utils.is_valid_time('00'))
        self.assertTrue(time_utils.is_valid_time('00:00'))
        self.assertTrue(time_utils.is_valid_time('1'))
        self.assertTrue(time_utils.is_valid_time('23'))
        self.assertTrue(time_utils.is_valid_time('23:59'))
        self.assertTrue(time_utils.is_valid_time('24'))
        self.assertTrue(time_utils.is_valid_time('24:00'))

    def test_parse_time(self):
        self.assertEqual(None, time_utils.parse_time(''))
        self.assertEqual(None, time_utils.parse_time('  '))

        self.assertEqual(time(0, 0), time_utils.parse_time('00:00'))
        self.assertEqual(time(5, 0), time_utils.parse_time('5'))
        self.assertEqual(time(10, 0), time_utils.parse_time('10:00'))
        self.assertEqual(time(14, 5), time_utils.parse_time('14:05'))
        self.assertEqual(time(0, 0), time_utils.parse_time('24:00'))

        self.assertRaises(ValueError, time_utils.parse_time, 'zz')
        self.assertRaises(ValueError, time_utils.parse_time, '01:60')
        self.assertRaises(ValueError, time_utils.parse_time, '-1')
        self.assertRaises(ValueError, time_utils.parse_time, '25')
        self.assertRaises(ValueError, time_utils.parse_time, '24:01')

    def test_parse_interval(self):
        self.assertEqual((time(10, 0), time(14, 0)), time_utils.parse_time_interval('10:00 - 14:00'))
        self.assertEqual((time(10, 0), time(0, 0)), time_utils.parse_time_interval('10:00 - 24:00'))
        self.assertEqual((time(10, 0), time(0, 0)), time_utils.parse_time_interval('10:00 - 00:00'))

        # -> read description, why it is so
        self.assertEqual((time(0, 0), time(0, 0)), time_utils.parse_time_interval('00:00 - 00:00'))
        self.assertEqual((time(0, 0), time(0, 0)), time_utils.parse_time_interval('24:00 - 00:00'))

        self.assertRaises(ValueError, time_utils.parse_time_interval, '10:33 - 10:33')
        self.assertRaises(ValueError, time_utils.parse_time_interval, '11:00 - 10:00')

    def test_parse_date(self):
        self.assertEqual(date(2019, 6, 17), time_utils.parse_date('2019-06-17'))
        self.assertEqual(date(2019, 3, 17), time_utils.parse_date('17.03.2019'))
        self.assertEqual(date(2019, 3, 2).day, time_utils.parse_date('02/03').day)
        self.assertEqual(date(2019, 3, 2).month, time_utils.parse_date('02/03').month)

        now = date.today()
        self.assertEqual(now + timedelta(0 - now.weekday()), time_utils.parse_date('Mo'))
        self.assertEqual(now + timedelta(1 - now.weekday()), time_utils.parse_date('вт'))
        self.assertEqual(now + timedelta(2 - now.weekday()), time_utils.parse_date('ср'))
        self.assertEqual(now + timedelta(3 - now.weekday()), time_utils.parse_date('th'))
        self.assertEqual(now + timedelta(4 - now.weekday()), time_utils.parse_date('Fr'))
        self.assertEqual(now + timedelta(5 - now.weekday()), time_utils.parse_date('sa'))
        self.assertEqual(now + timedelta(6 - now.weekday()), time_utils.parse_date('вс'))

        self.assertRaises(ValueError, time_utils.parse_date, 'четверг')
        self.assertRaises(ValueError, time_utils.parse_date, '2019=06=17')
        self.assertRaises(ValueError, time_utils.parse_date, '2019')
        self.assertRaises(ValueError, time_utils.parse_date, '2019.03')
        self.assertRaises(ValueError, time_utils.parse_date, '209-06-17')
        self.assertRaises(ValueError, time_utils.parse_date, '2200-06-17')

    def test_parse_date_interval(self):
        self.assertEqual((date(2019, 6, 17), date(2019, 6, 20)), time_utils.parse_date_interval('2019-06-17 - 2019-06-20'))
        self.assertEqual((date(date.today().year, 5, 20), date(date.today().year, 5, 21)), time_utils.parse_date_interval('20.05 - 21.05'))
        self.assertEqual((date(2019, 12, 30), date(2020, 1, 3)), time_utils.parse_date_interval('2019-12-30 - 2020-01-03'))
        self.assertEqual((date(2001, 4, 28), date(2001, 4, 28)), time_utils.parse_date_interval('28.04.2001'))

        start, end = time_utils.parse_date_interval('Mo - Fr')
        self.assertEqual(timedelta(days=4), end - start)
        start, end = time_utils.parse_date_interval('we - we')
        self.assertEqual(timedelta(days=7), end - start)
        start, end = time_utils.parse_date_interval('sa - mo')
        self.assertEqual(timedelta(days=2), end - start)

        self.assertRaises(ValueError, time_utils.parse_date_interval, '2019.03.21 - 2019.03.19')
        self.assertRaises(ValueError, time_utils.parse_date_interval, '01.21 - 01.19')
        self.assertRaises(ValueError, time_utils.parse_date_interval, '28.12 - 02.01')
        self.assertRaises(ValueError, time_utils.parse_date_interval, '2019-09-20 - 2019-10-10')



from typing import Union, List, Tuple

from PIL import ImageFont, ImageDraw, Image


class StringUtils:
    @staticmethod
    def is_blank(value: str) -> bool:
        return not (value and value.strip())

    @staticmethod
    def is_not_blank(value: str) -> bool:
        return bool(value and value.strip())

    @staticmethod
    def strip_lines(input_lines: List[str], strip_lines: bool) -> List[str]:
        result: list[str] = []
        for line in input_lines:
            for row in line.split('\n'):
                if strip_lines:
                    row = row.strip()
                result.append(row)
        return result

    @staticmethod
    def count_max_text_width(text: Union[str, List[str]], left_strip_lines: bool = False) -> int:
        if isinstance(text, str):
            return max([len(StringUtils.strip(line, left_strip_lines, True)) for line in text.split('\n')])
        if isinstance(text, list):
            return max([StringUtils.count_max_text_width(line, left_strip_lines) for line in text])
        raise ValueError(f'Wrong input type. Required: Union[str, list[str]]. Actual: {type(text)}')

    @staticmethod
    def strip(text: str, left_strip: bool, right_strip: bool) -> str:
        if left_strip:
            text = text.lstrip()
        if right_strip:
            text = text.rstrip()
        return text


class FontUtils:
    @staticmethod
    def get_text_size(font: ImageFont, text: str) -> Tuple[int, int]:
        if hasattr(font, 'getsize'):
            return font.getsize(text)

        # More information: https://pillow.readthedocs.io/en/stable/deprecations.html
        left, top, right, bottom = font.getbbox(text)
        return right - left, bottom - top

    @staticmethod
    def get_multiline_text_size(font: ImageFont, text: str) -> Tuple[int, int]:
        if hasattr(font, 'getsize_multiline'):
            return font.getsize_multiline(text)

        dummy_draw: ImageDraw = ImageDraw.Draw(Image.new('RGB', (1, 1)))
        # More information: https://pillow.readthedocs.io/en/stable/deprecations.html
        left, top, right, bottom = dummy_draw.multiline_textbbox((0, 0), text, font=font)
        return right - left, bottom - top
